# Template.pm - The Object Class that provides a Template Language Object
#$Id: Template.pm,v 1.11 2004/02/19 21:03:54 james Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Language::Template;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '1';

# The Phrases used by the Portal.  - Add these to the Template and convert in each language as created.

my %phrases;
%phrases = ();
$phrases{clientUserName} = "Client User Name";
$phrases{password} = "Password";
$phrases{unameNeeded} = "uname must be specified!";
$phrases{passwdNeeded} = "passwd must be specifed!";
$phrases{login} = "Login";
$phrases{stateNeeded} = "The State of the Application must be specified!";
$phrases{commandNeeded} = "The Command to execute must be specified!";
$phrases{sessionCookieNeeded} = "The Session Cookie must be defined!";
$phrases{sessionNotCreated} = "The Session does not exist but Session Cookie is defined!";
$phrases{inClientMode} = "In Client Mode.<br>\n";
$phrases{inNormalMode} = "In Normal Mode.<br>\n";
$phrases{commandNotKnown} = "Command = '%s' is not known!";
$phrases{stateNotKnown} = "State = '%s' is not known!";
$phrases{stateEvalFailed} = "Eval of State = '%s' failed!<br>\nError = '%s'.";
$phrases{appStateCommand} = "App = '%s', State = '%s', Command = '%s'.";
$phrases{invalidPassword} = "Password is invalid!";
$phrases{userNotActive} = "User = '%s' is not active!";
$phrases{userNotFound} = "Could not find User = '%s'!";
$phrases{logout} = "Logout";
$phrases{help} = "Help";
$phrases{appVersion} = "Accounting Version = <font color=\"%s\">%s</font>";
$phrases{run} = "Run";
$phrases{print} = "Print";
$phrases{refresh} = "Refresh";
$phrases{portfolioCustomizationTitle} = "Portfolio Customization Screen";
$phrases{glcCustomizationTitle} = "Capital Gain/Loss Customization Screen";
$phrases{execute} = "Execute";
$phrases{investmentAccounts} = "Investment Accounts";
$phrases{stocks} = "Stocks";
$phrases{assets} = "Assets";
$phrases{date} = "Date";
$phrases{startDate} = "Start Date";
$phrases{endDate} = "End Date";
$phrases{selectAll} = "Select All";
$phrases{unSelectAll} = "Un-Select All";
$phrases{unSelect} = "Un-Select";
$phrases{ledgerRequired} = "ledger required!<br>\n";
$phrases{invalidLedger} = "ledger = '%s' is invalid!<br>\n";
$phrases{portfolioPhrase} = "Portfolio";
$phrases{negativeNumberFormatPhrase} = "Negative Number Format";
$phrases{stockPriceTitle} = "Stock Price Maintenance";
$phrases{stockPrices} = "Stock Prices";
$phrases{investmentJournal} = "Investment Journal";
$phrases{watchedStocks} = "Watched Stocks";
$phrases{dateInfo} = "Historical Date Info";
$phrases{reset} = "Reset";
$phrases{updateWatchedStocks} = "Update Watched Stocks";
$phrases{selectStockToWatch} = "You must select a Stock to Watch!";
$phrases{selectStockToUnWatch} = "You must select a Stock to Un-Watch!";
$phrases{alreadyWatchingStock} = "You are already Watching Stock";
$phrases{here} = "here";
$phrases{noStocksCreated} = "No Stocks created!";
$phrases{noTickeredStocksCreated} = "No Stocks created with Tickers!";
$phrases{go} = "Go";
$phrases{toCreateStocks} = "to create Stocks.";
$phrases{moreWatchedStocksReturned} = "More watched Stocks returned than Stocks are created!";
$phrases{toMaintainStocks} = "to maintain Stocks.";
$phrases{maintainStocks} = "Maintain Stocks";
$phrases{updateCurrentStockPrice} = "Update Current Stock Price";
$phrases{updateHistoricalStockPrice} = "Update Historical Stock Price";
$phrases{day} = "day";
$phrases{week} = "week";
$phrases{month} = "month";
$phrases{startDate} = "Start Date";
$phrases{endDate} = "End Date";
$phrases{interval} = "Interval";
$phrases{stockPriceInstructions} = "To select stocks to update prices for (current or historical) choose from the <b>Stocks</b> select box. &nbsp;If doing Historical prices, select the Start and End Date values and the Interval to group by and then Click Go!";
$phrases{orderBy} = "Order By";
$phrases{displayZeroShares} = "Display Zero Shares";
$phrases{create} = "Create";
$phrases{edit} = "Edit";
$phrases{delete} = "Delete";
$phrases{noEntriesFound} = "No Entries found";
$phrases{monitor} = "Monitor";
$phrases{account} = "Account";
$phrases{entry} = "Entry";
$phrases{id} = "Id";
$phrases{code} = "Code";
$phrases{ledger} = "Ledger";
$phrases{accounts} = "Accounts";
$phrases{emails} = "Emails";
$phrases{users} = "Users";
$phrases{operator} = "Operator";
$phrases{MailType} = "Mail Type";
$phrases{MailServer} = "Mail Server";
$phrases{for} = "for";
$phrases{errorExclam} = "Error!";
$phrases{missingArgument} = "<b>%s</b> is missing";
$phrases{invalidArgument} = "<b>%s</b> = '%s' is invalid";
$phrases{FromColon} = "From:";
$phrases{ToColon} = "To:";

# new 
# instantiates an instance of the Template Language Phrases.
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );

  $self->{phrases} = { %phrases };
  $self->{encoding} = "iso-8859-1";

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->setError($self->{errorString});
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.

  if (length $errorString > 0)
  {
    $error = 1;
    $self->setError($errorString);
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD  # act as though all calls are just queries, setting of values will not work.
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  return $self->{$name};
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "$subName$self->{errorPhrase}";

  if (scalar @_ == 1)
  {
    $self->{errorString} = $callerErrStr . @_[0];
  }
  else
  {
    my %args = ( @_ );
    if (!exists $args{errorString})  # make sure we get the errorString argument!
    {
      $self->setError("<b>errorString</b> is missing!<br>\n");
    }
    else
    {
      $self->{errorString} = $callerErrStr . $args{errorString};
    }
  }
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;
  
  return $self->{errorString};
}

1;
__END__

=head1 NAME

Template - Object used to build a Template Object Class.

=head1 SYNOPSIS

  use Portal::Language::Template;
  my $obj = Portal::Language::Template->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Template is a Template Language class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::Language::Template
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported VARIABLES

  hash   phrases  - converted phrases
  scalar encoding - content type encoding for document.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
