# GlobalSecurityEventObject.pm - The Object Class that provides a Global Security Event Object
# that contains the necessary data values based on the event code
# Created by Jason A. Pattie, 12/01/2000.
#$Id: GlobalSecurityEventObject.pm,v 1.1.1.1 2001/08/09 19:04:51 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::GlobalSecurityEventObject;
use strict;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a GlobalSecurityEventObject
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my $errorHeader = "GlobalSecurityEventObject->new(): ";
  my %args = (eventCode => undef, calcDate => undef, transDate => undef, newTicker => undef, oldTicker => undef,
              newName => undef, oldName => undef, memoText => undef, subAccountNum => undef,
              subAccountNum2 => undef, amount => undef, newShares => undef, oldShares => undef,
              costPercentage => undef, eventRefNum => undef, @_ );

  $self->{error} = 0;  # nothing wrong yet.
  $self->{errorString} = "";

  $self->{eventCode}      = $args{'eventCode'};
  $self->{calcDate}       = $args{'calcDate'};
  $self->{transDate}      = $args{'transDate'};
  $self->{newTicker}      = $args{'newTicker'};
  $self->{oldTicker}      = $args{'oldTicker'};
  $self->{newName}        = $args{'newName'};
  $self->{oldName}        = $args{'oldName'};
  $self->{memoText}       = $args{'memoText'};
  $self->{subAccountNum}  = $args{'subAccountNum'};
  $self->{subAccountNum2} = $args{'subAccountNum2'};
  $self->{amount}         = $args{'amount'};
  $self->{newShares}      = $args{'newShares'};
  $self->{oldShares}      = $args{'oldShares'};
  $self->{costPercentage} = $args{'costPercentage'};

  return $self;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  my $errorHeader = "GlobalSecurityEventObject->isValid(): ";
  my @validate_list = ();

  if ($self->{eventCode} eq "90") # stock split
  {
    @validate_list = ("transDate", "subAccountNum", "oldShares", "newShares", "costPercentage", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "100") # stock dividend
  {
    @validate_list = ("calcDate", "transDate", "subAccountNum", "oldShares", "newShares", "costPercentage", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "60") # stock swap
  {
    @validate_list = ("calcDate", "transDate", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "costPercentage", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "80") # acquisition
  {
    @validate_list = ("calcDate", "transDate", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "costPercentage", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "70") # spin off
  {
    @validate_list = ("calcDate", "transDate", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "costPercentage", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "memo")
  {
    @validate_list = ("transDate", "memoText", "subAccountNum", "ledgerCode", "eventRefNum", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "50") # return of capital
  {
    @validate_list = ("transDate", "subAccountNum", "amount", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "tchn")
  {
    @validate_list = ("transDate", "newTicker", "oldTicker", "subAccountNum", "eventRefNum");
  }
  elsif ($self->{eventCode} eq "nchn")
  {
    @validate_list = ("transDate", "newName", "oldName", "subAccountNum", "eventRefNum");
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid eventCode assigned to this object (" . $self->{eventCode} . ")!";
    return 0;
  }
  # run through the validate list and validate each necessary field value
  for (my $validatecount=0; $validatecount < scalar @validate_list; $validatecount++)
  {
    if (not defined $self->{$validate_list[$validatecount]} or $self->{$validate_list[$validatecount]} eq "")
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "The $validate_list[$validatecount] does not have a value and must be specified!";
      return 0;
    }
    if ($validate_list[$validatecount] eq "calcDate")
    {
      if ($self->{calcDate} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty calc date!  The calc date must be specified.";
        return 0;
      }
      if ($self->{calcDate} !~ /([0][1-9]|[1][0-2])[-\/]([0][1-9]|[1-2][0-9]|[3][0-1])[-\/]([0-9]{4})/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid date format (" . $self->{calcDate} . ")!  The calc date must be of the form MM/DD/YYYY or MM-DD-YYYY.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "transDate")
    {
      if ($self->{transDate} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty trans date!  The trans date must be specified.";
        return 0;
      }
      if ($self->{transDate} !~ /([0][1-9]|[1][0-2])[-\/]([0][1-9]|[1-2][0-9]|[3][0-1])[-\/]([0-9]{4})/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid date format ( " . $self->{transDate} . ")!  The trans date must be of the form MM/DD/YYYY or MM-DD-YYYY.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "memoText")
    {
      if ($self->{memoText} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty memo text!  The memo text must be specified.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "subAccountNum")
    {
      if ($self->{subAccountNum} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty sub-account number!  The sub-account number must be specified.";
        return 0;
      }
      if ($self->{subAccountNum} !~ /^-?[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "The sub-account number does not appear to be numeric (" . $self->{subAccountNum} . ")!";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "subAccountNum2")
    {
      if ($self->{subAccountNum2} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty sub-account number 2!  The sub-account number 2 must be specified.";
        return 0;
      }
      if ($self->{subAccountNum2} !~ /^-?[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "The sub-account number 2 does not appear to be numeric (" . $self->{subAccountNum2} . ")!";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "amount")
    {
      if ($self->{amount} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty amount!  The amount must be specified.";
        return 0;
      }
      if ($self->{amount} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid amount (" . $self->{amount} . ")!  Amount does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "newShares")
    {
      if ($self->{newShares} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty new shares!  The new shares must be specified.";
        return 0;
      }
      if ($self->{newShares} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid new shares (" . $self->{newShares} . ")!  New shares does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "oldShares")
    {
      if ($self->{oldShares} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty old shares!  The old shares must be specified.";
        return 0;
      }
      if ($self->{oldShares} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid old shares (" . $self->{oldShares} . ")!  Old shares does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "costPercentage")
    {
      if ($self->{costPercentage} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid cost percentage (" . $self->{commission} . ")!  Cost percentage does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "eventRefNum")
    {
      if ($self->{eventRefNum} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid event reference number!  Event reference number does not appear to be numeric.";
        return 0;
      }
    }
  }

  return 1;
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

GlobalSecurityEventObject - Object used to build a global security event that can be used for database transactions.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::GlobalSecurityEventObject;
  my $globalsecobj = GlobalSecurityEventObject->new(eventCode => 'spof',
                                        calcDate => '06-29-2000',
                                        transDate => '07-02-2000',
                                        subAccountNum => 1500,
                                        subAccountNum2 => 1605,
                                        oldShares => 2,
                                        newShares => 5,
                                        costPercentage => 0.25);
  if ($globalsecobj->didErrorOccur())
  {
    die $globalsecobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $globalsecobj->eventCode("memo");
  $globalsecobj->memoText("Spinoff of stock happened Tuesday.");

  # make sure the object is useful by calling the validation routine
  # before writing it to the database
  if ($globalsecobj->isValid())
  {
    ...
  }
  else
  {
    die $globalsecobj->errorMessage();
  }

=head1 DESCRIPTION

GlobalSecurityEventObject is a class that stores data for a global
security event transaction.  It provides a container for a global
security event that will be used to write to the global_sec_event_tb
table in the database.

=head1 Exported FUNCTIONS

  scalar new(eventCode, calcDate, transDate, memoText,
             subAccountNum, subAccountNum2, amount,
             newShares, oldShares, costPercentage)
    Creates a new instance of the
    Portal::Accounting::Objects::GlobalSecurityEventObject
    object.  A new instance can be created in an invalid state
    (i.e., missing fields, etc.).  isValid() must be called on
    the object before using the object in a database transaction.

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function should be called before
    using this object in a database transaction.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified and
         passing the "method" the value to assign as the first
         argument.

=head1 AUTHOR

Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut